﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using Microsoft.Extensions.Primitives;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;

using System.ComponentModel.DataAnnotations;
using System.Collections;
using Syncfusion.EJ2.Linq;

namespace AngularwithASPCore.Controllers
{
    [Produces("application/json")]
    [Route("api/Orders")]

    public class OrdersController : Controller
    {
        // public static List<SelfReferenceData> FlatData = new List<SelfReferenceData>();
        public static List<SelfReferenceData> tree = new List<SelfReferenceData>();
        // GET: api/Orders
        [HttpGet]

        public object Get()
        {

            var queryString = Request.Query;
            if (tree.Count == 0)
                tree = SelfReferenceData.GetTree();
            if (queryString.Keys.Contains("$filter") && !queryString.Keys.Contains("$top"))
            {
                StringValues filter;
                queryString.TryGetValue("$filter", out filter);
                int fltr = Int32.Parse(filter[0].ToString().Split("eq")[1]);
                IQueryable<SelfReferenceData> data1 = tree.Where(f => f.ParentItem == fltr).AsQueryable();
                return new { result = data1.ToList(), count = data1.Count() };
            }
            List<SelfReferenceData> data = tree.ToList();
            if (queryString.Keys.Contains("$select"))
            {
                data = (from ord in tree
                        select new SelfReferenceData
                        {
                            ParentItem = ord.ParentItem
                        }
                        ).ToList();
                return data;
            }
            data = data.Where(p => p.ParentItem == null).ToList();
            int count = data.Count;

            if (queryString.Keys.Contains("$inlinecount"))
            {
                StringValues Skip;
                StringValues Take;
                StringValues loadchild;
                

                int skip = (queryString.TryGetValue("$skip", out Skip)) ? Convert.ToInt32(Skip[0]) : 0;
                int top = (queryString.TryGetValue("$top", out Take)) ? Convert.ToInt32(Take[0]) : data.Count();
                
                    var GroupData = SelfReferenceData.GetTree().ToList().GroupBy(rec => rec.ParentItem)
                               .Where(g => g.Key != null).ToDictionary(g => g.Key?.ToString(), g => g.ToList());
                    foreach (var Record in tree.ToList())
                    {
                        if (GroupData.ContainsKey(Record.TaskID.ToString()))
                        {
                            var ChildGroup = GroupData[Record.TaskID.ToString()];
                            if (ChildGroup?.Count > 0)
                                AppendChildren(ChildGroup, Record, GroupData);
                        }
                    }
                
                return new { result = tree.Skip(skip).Take(top), count = tree.Count };
            }
            else
            {
                return SelfReferenceData.GetTree();
            }


        }
        private void AppendChildren(List<SelfReferenceData> ChildRecords, SelfReferenceData ParentItem, Dictionary<string, List<SelfReferenceData>> GroupData)
        {

            var queryString = Request.Query;
            string TaskId = ParentItem.TaskID.ToString();
            
                var index = tree.IndexOf(ParentItem);
                foreach (var Child in ChildRecords)
                {
                    string ParentId = Child.ParentItem.ToString();
                    if (TaskId == ParentId)
                    {
                        if (tree.IndexOf(Child) == -1)
                            ((IList)tree).Insert(++index, Child);
                        if (GroupData.ContainsKey(Child.TaskID.ToString()))
                        {
                            var DeepChildRecords = GroupData[Child.TaskID.ToString()];
                            if (DeepChildRecords?.Count > 0)
                                AppendChildren(DeepChildRecords, Child, GroupData);
                        }
                    }
                }
          

        }



        // GET: api/Orders/5
        [HttpGet("{id}", Name = "Get")]
        public string Get(int id)
        {
            return "value";
        }




        [HttpPost]
        public object Post([FromBody] SelfReferenceData value)
        {
            //handle insert action
            tree.Insert(0, value);
            return value;
        }

        //// PUT: api/Orders/5
        [HttpPut]

        public object Put(int id, [FromBody] SelfReferenceData value)
        {
            //handle edit action
            var ord = value;
            SelfReferenceData val = tree.Where(or => or.TaskID == ord.TaskID).FirstOrDefault();
            val.TaskID = ord.TaskID;
            val.TaskName = ord.TaskName;
            //val.CustomerID = ord.CustomerID;
            //val.Freight = ord.Freight;
            //val.OrderDate = ord.OrderDate;
            //val.ShipCity = ord.ShipCity;
            return value;
        }
        // DELETE: api/ApiWithActions/5
        [HttpDelete("{id:int}")]
        [Route("Orders/{id:int}")]
        public object Delete(int id)
        {
            //handle delete action
            tree.Remove(tree.Where(or => or.TaskID == id).FirstOrDefault());
            return Json(id);
        }
        public class SelfReferenceData
        {

            [Key]
            public int TaskID { get; set; }
            public string TaskName { get; set; }
            public DateTime StartDate { get; set; }
            public DateTime EndDate { get; set; }
            public String Progress { get; set; }
            public String Priority { get; set; }
            public int Duration { get; set; }
            public int? ParentItem { get; set; }
            public bool? isParent { get; set; }
            public SelfReferenceData() { }
            public static List<SelfReferenceData> GetTree()
            {
                if (tree.Count == 0)
                {
                    int root = -1;
                    for (var t = 1; t <= 1; t++)
                    {
                        Random ran = new Random();
                        string math = (ran.Next() % 3) == 0 ? "High" : (ran.Next() % 2) == 0 ? "Release Breaker" : "Critical";
                        string progr = (ran.Next() % 3) == 0 ? "Started" : (ran.Next() % 2) == 0 ? "Open" : "In Progress";
                        root++;
                        int rootItem = tree.Count + root + 1;
                        tree.Add(new SelfReferenceData() { TaskID = rootItem, TaskName = "Parent Task " + rootItem.ToString(), StartDate = new DateTime(1992, 06, 07), EndDate = new DateTime(1994, 08, 25), isParent = true, ParentItem = null, Progress = progr, Priority = math, Duration = ran.Next(1, 50) });
                        int parent = tree.Count;
                        //for (var c = 0; c < 10; c++)
                        //{
                        //    root++;
                        //    string val = ((parent + c + 1) % 3 == 0) ? "Low" : "Critical";
                        //    int parn = parent + c + 1;
                        //    progr = (ran.Next() % 3) == 0 ? "In Progress" : (ran.Next() % 2) == 0 ? "Open" : "Validated";
                        //    int iD = tree.Count + root + 1;
                        //    tree.Add(new SelfReferenceData() { TaskID = iD, TaskName = "Child Task " + iD.ToString(), StartDate = new DateTime(1992, 06, 07), EndDate = new DateTime(1994, 08, 25), isParent = (((parent + c + 1) % 3) == 0), ParentItem = rootItem, Progress = progr, Priority = val, Duration = ran.Next(1, 50) });
                        //    if ((((parent + c + 1) % 3) == 0))
                        //    {
                        //        int immParent = tree.Count;
                        //        for (var s = 0; s < 3; s++)
                        //        {
                        //            root++;
                        //            string Prior = (immParent % 2 == 0) ? "Validated" : "Normal";
                        //            tree.Add(new SelfReferenceData() { TaskID = tree.Count + root + 1, TaskName = "Sub Task " + (tree.Count + root + 1).ToString(), StartDate = new DateTime(1992, 06, 07), EndDate = new DateTime(1994, 08, 25), isParent = false, ParentItem = iD, Progress = (immParent % 2 == 0) ? "On Progress" : "Closed", Priority = Prior, Duration = ran.Next(1, 50) });
                        //        }
                        //    }
                        //}
                    }
                }
                return tree;
            }
        }


    }

}